package com.persagy.apm.energy.report.common.service.impl;

import com.persagy.apm.energy.report.common.dto.QueryPlatformParamDTO;
import com.persagy.apm.energy.report.common.service.CommonService;
import com.persagy.apm.energy.report.common.service.PlatformService;
import com.persagy.apm.energy.report.common.utils.DataUtils;
import com.persagy.apm.energy.report.common.utils.DateUtils;
import com.persagy.apm.energy.report.common.vo.PlatformAreaInfoVO;
import com.persagy.apm.energy.report.monthly.config.statisticsitems.model.StatisticItems;
import com.persagy.apm.energy.report.workorder.model.WorkOrder;
import com.persagy.apm.energy.report.workorder.repository.WorkOrderRepository;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.elasticsearch.core.query.NativeSearchQuery;
import org.springframework.data.elasticsearch.core.query.NativeSearchQueryBuilder;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.TreeMap;
import java.util.stream.Collectors;

@Service("OrderCompleteRateWorkOrderWebService")
@Slf4j
public class WorkOrderPlatformServiceImpl implements PlatformService {

    @Resource
    private CommonService commonService;
    @Autowired
    private WorkOrderRepository workOrderRepository;

    @Override
    public List<Double> getPjPlatformRate(QueryPlatformParamDTO queryPlatformParamDTO) {
        if (null == queryPlatformParamDTO || CollectionUtils.isEmpty(queryPlatformParamDTO.getProjectIdList()) || null == queryPlatformParamDTO.getReportDate()) {
            return new ArrayList<>();
        }
        String pjId = queryPlatformParamDTO.getProjectIdList().get(0);
        TreeMap<Date, Double> date_rateMap = new TreeMap<>();
        Date reportDate = queryPlatformParamDTO.getReportDate();
        Date startDate = DateUtils.getFirstDayOfYear(reportDate);
        while (startDate.before(DateUtils.getMonthOff(reportDate, 1))) {
            date_rateMap.put(startDate, getWorkOrderCompleteRate(pjId, startDate));
            startDate = DateUtils.getMonthOff(startDate, 1);
        }
        List<Double> monthlySummaries = date_rateMap.values().stream().collect(Collectors.toList());
        return monthlySummaries;
    }

    private Double getWorkOrderCompleteRate(String projectId, Date reportDate) {
        Double completeRate = null;
        try {
            Integer workCount = getWorkOrderNumByParam(projectId, reportDate, "create_time", null);
            Integer dealCount = getWorkOrderNumByParam(projectId, reportDate, "ask_end_time", "8");
            if (null != workCount && workCount != 0 && null != dealCount) {
                completeRate = DataUtils.parseDouble(dealCount) / workCount * 100;
            }
            return completeRate;
        } catch (Exception e) {
            log.error("计算工单完成率异常", e);
        }
        return completeRate;
    }

    private Integer getWorkOrderNumByParam(String projectId, Date reportDate, String timeType, String orderState) {
        try {
            String startTime = DateUtils.date2Str(reportDate, DateUtils.SDFSECOND);
            String endTime = DateUtils.date2Str(DateUtils.getMonthOff(reportDate, 1), DateUtils.SDFSECOND);
            BoolQueryBuilder queryOrderNumBuilder = createQueryParam(projectId, timeType, startTime, endTime, orderState);
            NativeSearchQueryBuilder nativeSearchQueryBuilder = new NativeSearchQueryBuilder();
            nativeSearchQueryBuilder.withQuery(queryOrderNumBuilder);
            NativeSearchQuery query = nativeSearchQueryBuilder.build();
            Page<WorkOrder> resultList = workOrderRepository.search(query);
            if (null != resultList) {
                return resultList.getSize();
            }
        } catch (Exception e) {
            log.error("获取工单数据异常", e);
        }
        return null;
    }

    private BoolQueryBuilder createQueryParam(String pjId, String timeType, String startTime, String endTime, String orderState) {
        BoolQueryBuilder queryBuilder = QueryBuilders.boolQuery();
        queryBuilder.must(QueryBuilders.matchQuery("project_id", pjId));
        //时间查询
        if (StringUtils.isNotBlank(timeType)) {
            if (StringUtils.isNotBlank(startTime)) {
                if ("create_time".equals(timeType)) {
                    queryBuilder.must(QueryBuilders.rangeQuery("create_time").gte(startTime));
                } else if ("ask_end_time".equals(timeType)) {
                    queryBuilder.must(QueryBuilders.rangeQuery("ask_end_time").gte(startTime));
                }
            }
            if (StringUtils.isNotBlank(endTime)) {
                if ("create_time".equals(timeType)) {
                    queryBuilder.must(QueryBuilders.rangeQuery("create_time").lt(endTime));
                } else if ("ask_end_time".equals(timeType)) {
                    queryBuilder.must(QueryBuilders.rangeQuery("ask_end_time").lt(endTime));
                }
            }
        }
        if (StringUtils.isNotEmpty(orderState)) {
            queryBuilder.must(QueryBuilders.matchQuery("order_state", orderState));
        }
        return queryBuilder;
    }

    @Override
    public PlatformAreaInfoVO getAreaPlatformRate(StatisticItems statisticItems, QueryPlatformParamDTO queryPlatformParamDTO) {
        if (null == queryPlatformParamDTO || CollectionUtils.isEmpty(queryPlatformParamDTO.getProjectIdList()) || null == queryPlatformParamDTO.getReportDate()) {
            return new PlatformAreaInfoVO();
        }
        // 合格标准
        String standardValueStr = statisticItems.getStandardValue();
        Double standardValue = DataUtils.parseDouble(standardValueStr);
        // 校验规则
        String qualifyFormula = statisticItems.getQualifyFormula();
        // 达标数量
        Integer satisfiedCount = 0;
        // 未达标数量
        Integer unsatisfiedCount = 0;
        // 未达标项目id列表
        List<String> unsatisfiedPjIdList = new ArrayList<>();
        for (String pjId : queryPlatformParamDTO.getProjectIdList()) {
            Double completeRate = getWorkOrderCompleteRate(pjId, queryPlatformParamDTO.getReportDate());
            if (null != completeRate && null != standardValue) {
                if (DataUtils.getQualifyResult(standardValue, qualifyFormula, completeRate) == 0.0) {
                    unsatisfiedCount++;
                    unsatisfiedPjIdList.add(pjId);
                } else {
                    satisfiedCount++;
                }
            }
        }
        // 根据未达标项目id列表获取不达标项目名称
        List<String> pjNameList = commonService.getPjNameByPjId(unsatisfiedPjIdList);
        PlatformAreaInfoVO platformAreaInfoVO = new PlatformAreaInfoVO();
        platformAreaInfoVO.setSatisfiedCount(satisfiedCount);
        platformAreaInfoVO.setUnsatisfiedCount(unsatisfiedCount);
        platformAreaInfoVO.setUnsatisfiedProjects(pjNameList);
        return platformAreaInfoVO;
    }
}
