package com.persagy.apm.energy.report.monthly.detail.common.service.impl;

import com.persagy.apm.energy.report.common.utils.DataUtils;
import com.persagy.apm.energy.report.common.utils.DateUtils;
import com.persagy.apm.energy.report.monthly.config.function.constant.enums.FunctionTypeEnum;
import com.persagy.apm.energy.report.monthly.config.function.model.Function;
import com.persagy.apm.energy.report.monthly.config.function.service.IFunctionService;
import com.persagy.apm.energy.report.monthly.config.functiongroup.model.FunctionGroup;
import com.persagy.apm.energy.report.monthly.config.functiongroup.service.IFunctionGroupService;
import com.persagy.apm.energy.report.monthly.detail.common.model.vo.CostItemVO;
import com.persagy.apm.energy.report.monthly.detail.common.model.vo.CostVO;
import com.persagy.apm.energy.report.monthly.detail.common.model.vo.GroupInfo;
import com.persagy.apm.energy.report.monthly.detail.common.service.IReportCostInfoService;
import com.persagy.apm.energy.report.monthly.functionvalue.service.IFunctionValueService;
import com.persagy.apm.energy.report.saasweb.model.vo.ReportProjectVO;
import com.persagy.apm.energy.report.saasweb.service.ISaasWebService;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 商业类报告费用service类
 *
 * @author lixing
 * @version V1.0 2021/5/30 7:30 下午
 **/
@Service
public abstract class ReportCostInfoServiceImpl implements IReportCostInfoService {
    @Autowired
    private IFunctionGroupService functionGroupService;
    @Autowired
    private IFunctionService functionService;
    @Autowired
    private IFunctionValueService functionValueService;
    @Autowired
    private ISaasWebService saasWebService;

    @Override
    public CostVO getCostInfoByProjectIds(List<String> groupIdList, List<String> functionIdList, Date reportMonth, List<String> projectIds) {
        CostVO costVO = new CostVO();
        List<GroupInfo<CostItemVO>> costGroupInfos = getCostGroupInfos(groupIdList, reportMonth, projectIds);
        costVO.setGroups(costGroupInfos);

        List<CostItemVO> costItemInfos = getCostItemInfos(functionIdList, reportMonth, projectIds);
        costVO.setItems(costItemInfos);

        List<CostItemVO> paragraphSums = new ArrayList<>();

        if (!CollectionUtils.isEmpty(costGroupInfos)) {
            paragraphSums.addAll(
                    costGroupInfos.stream().map(GroupInfo::getSummary).collect(Collectors.toList()));
        }
        if (!CollectionUtils.isEmpty(costItemInfos)) {
            paragraphSums.addAll(costItemInfos);
        }
        costVO.setSummary(getCostSum(paragraphSums));
        return costVO;
    }

    @Override
    public List<CostItemVO> getCostItemInfos(List<String> functionIdList, Date reportMonth, List<String> projectIds) {
        List<CostItemVO> costItemVOList = Lists.newArrayList();
        if (CollectionUtils.isEmpty(functionIdList) || CollectionUtils.isEmpty(projectIds)) {
            return costItemVOList;
        }

        for (String functionId : functionIdList) {
            Function function = functionService.queryFunctionDetail(functionId);
            String itemName = functionService.getItemName(function, projectIds.get(0));

            List<CostItemVO> projectCostItems = getCostItemsByProjects(reportMonth, projectIds, function);
            CostItemVO costSumByProjects = getCostSum(projectCostItems);
            costSumByProjects.setName(itemName);
            costSumByProjects.setEnergyType(function.getEnergyType());
            costSumByProjects.setFunctionType(function.getFunctionType());
            costItemVOList.add(costSumByProjects);
        }
        return costItemVOList;
    }

    @Override
    public List<CostItemVO> getCostItemsByProjects(Date reportMonth, List<String> projectIds, Function function) {
        List<CostItemVO> projectCostItems = Lists.newArrayList();
        if (CollectionUtils.isEmpty(projectIds)) {
            return projectCostItems;
        }

        // 统计项目总的商业面积
        Double projectCostBuildingArea = getProjectBuildingArea(projectIds);

        for (String projectId : projectIds) {
            // 获取项目信息
            ReportProjectVO reportProjectInfo = saasWebService.getReportProjectInfo(projectId);
            if (reportProjectInfo == null) {
                continue;
            }
            CostItemVO costItemVO = getCostItemInfo(reportMonth, projectId, projectCostBuildingArea, function);
            projectCostItems.add(costItemVO);
        }
        return projectCostItems;
    }

    /**
     * 获取统计费用需要使用的项目面积
     *
     * @param projectIds 需要统计的项目的id列表
     * @return 统计费用需要使用的项目面积
     * @author lixing
     * @version V1.0 2021/6/9 7:51 下午
     */
    @Override
    public abstract Double getProjectBuildingArea(List<String> projectIds);

    @Override
    public CostItemVO getCostItemInfo(
            Date reportMonth, String projectId,
            Double costBuildingArea, Function function) {
        if (function == null) {
            return null;
        }
        String functionId = function.getId();
        String functionType = function.getFunctionType();
        // 单位
        double unit = 1d;
        // 费用展示为万元
        if (FunctionTypeEnum.COST.getType().equals(functionType)) {
            unit = 10000d;
        }

        CostItemVO costItemVO = new CostItemVO();
        Double currentMonth = functionValueService.getFunctionValue(functionId, reportMonth, projectId);
        if (currentMonth != null) {
            costItemVO.setCurrentMonth(DataUtils.doubleDivide(currentMonth, unit));
        }

        Double lastMonth = functionValueService.getFunctionValue(
                functionId, DateUtils.getFirstDayOfLastMonth(reportMonth), projectId);
        if (lastMonth != null) {
            costItemVO.setLastMonth(DataUtils.doubleDivide(lastMonth, unit));
        }

        // 统计去年同期累计费用
        Double lastYearCount = null;
        Date lastYearSameTime = DateUtils.getSameMonthFirstDayOfLastYear(reportMonth);
        List<Date> lastYearMonths = DateUtils.getFirstDayOfEveryMonth(
                DateUtils.getFirstDayOfLastYear(reportMonth),
                lastYearSameTime);
        List<CostItemVO> lastYearItems = new ArrayList<>();
        for (Date lastYearMonth : lastYearMonths) {
            CostItemVO tmp = new CostItemVO();
            tmp.setLastYearCount(functionValueService.getFunctionValue(functionId, lastYearMonth, projectId));
            lastYearItems.add(tmp);
        }
        Double zero = 0d;
        CostItemVO lastYearSumItem = getCostSum(lastYearItems);
        if (lastYearSumItem != null) {
            lastYearCount = lastYearSumItem.getLastYearCount();
            if (lastYearCount != null) {
                costItemVO.setLastYearCount(DataUtils.doubleDivide(lastYearCount, unit));
            }
            if (costBuildingArea != null && !costBuildingArea.equals(zero) && lastYearCount != null) {
                costItemVO.setLastYearPerCount(DataUtils.doubleDivide(lastYearCount, costBuildingArea));
            }
        }

        // 统计今年的费用
        Double yearCount = null;
        List<Date> yearMonths = DateUtils.getFirstDayOfEveryMonth(
                DateUtils.getFirstDayOfYear(reportMonth),
                reportMonth);
        List<CostItemVO> yearItems = new ArrayList<>();
        for (Date yearMonth : yearMonths) {
            CostItemVO tmp = new CostItemVO();
            tmp.setYearCount(functionValueService.getFunctionValue(functionId, yearMonth, projectId));
            yearItems.add(tmp);
        }

        CostItemVO yearSumItem = getCostSum(yearItems);
        if (yearSumItem != null) {
            yearCount = yearSumItem.getYearCount();
            if (yearCount != null) {
                costItemVO.setYearCount(DataUtils.doubleDivide(yearCount, unit));
            }
            if (costBuildingArea != null && !costBuildingArea.equals(zero) && yearCount != null) {
                costItemVO.setYearPerCount(DataUtils.doubleDivide(yearCount, costBuildingArea));
            }
        }

        // 环比
        if (currentMonth != null && lastMonth != null) {
            Double linkCount = DataUtils.doubleSubtract(currentMonth, lastMonth);
            costItemVO.setLinkCount(DataUtils.doubleDivide(linkCount, unit));
            if (!lastMonth.equals(zero)) {
                Double linkRange = DataUtils.doubleDivide(linkCount, lastMonth);
                costItemVO.setLinkRange(linkRange);
            }
        }

        // 累计同比
        if (lastYearCount != null && yearCount != null) {
            double sameCount = DataUtils.doubleSubtract(yearCount, lastYearCount);
            costItemVO.setSameCount(DataUtils.doubleDivide(sameCount, unit));
            if (!lastYearCount.equals(zero)) {
                costItemVO.setSameRange(DataUtils.doubleDivide(sameCount, lastYearCount));
            }
        }

        // 去年同期月
        Double lastYearSameTimeValue = functionValueService.getFunctionValue(functionId, lastYearSameTime, projectId);

        if (lastYearSameTimeValue != null && currentMonth != null) {
            double sameTimeGrowth = DataUtils.doubleSubtract(currentMonth, lastYearSameTimeValue);
            // 月同比增量
            costItemVO.setSameTimeGrowth(sameTimeGrowth);
            if (!lastYearSameTimeValue.equals(zero)) {
                // 月同比增幅
                costItemVO.setSameTimeRange(DataUtils.doubleDivide(sameTimeGrowth, lastYearSameTimeValue));
            }
        }

        return costItemVO;
    }


    @Override
    public CostItemVO getCostSum(List<CostItemVO> costItems) {
        return DataUtils.sumDoubleParams(costItems, CostItemVO.class);
    }


    @Override
    public List<GroupInfo<CostItemVO>> getCostGroupInfos(List<String> groupIdList, Date reportMonth, List<String> projectIds) {
        List<GroupInfo<CostItemVO>> groupInfoList = Lists.newArrayList();
        if (CollectionUtils.isEmpty(groupIdList)) {
            return groupInfoList;
        }
        for (String groupId : groupIdList) {
            GroupInfo<CostItemVO> groupInfo = new GroupInfo<>();
            FunctionGroup functionGroup = functionGroupService.queryFunctionGroupDetail(groupId);
            groupInfo.setGroupName(functionGroup.getName());
            // 获取组下的分组（本期不做）
            // 获取组下的分项
            String functionIdStr = functionGroup.getFunctionIds();
            List<String> functionIds = Lists.newArrayList(functionIdStr.split(","));
            List<CostItemVO> costItemInfos = getCostItemInfos(functionIds, reportMonth, projectIds);
            groupInfo.setItems(costItemInfos);
            groupInfo.setSummary(getCostSum(costItemInfos));
            groupInfoList.add(groupInfo);
        }
        return groupInfoList;
    }
}
